/*
 * Copyright (c) 2007, University of Wisconsin-Madison
 *
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 * 
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * 
 *
 *      Tim Czerwonka                   Reseacher
 *      timc@cs.wisc.edu                Computer Sciences Department
 *      http://www.cs.wisc.edu/~timc    University of Wisconsin - Madison
 *      phone: 608-262-6610             1210 West Dayton Street
 *                                      Madison, WI 53706
 *
 */

/*
 * 
 * $Id: uwkfuncs.cpp,v 1.7 2007/12/17 21:17:57 timc Exp $
 * 
 * 
 * $Log: uwkfuncs.cpp,v $
 * Revision 1.7  2007/12/17 21:17:57  timc
 * *** empty log message ***
 *
 * Revision 1.6  2007/12/17 21:09:48  timc
 * license
 *
 * Revision 1.5  2007/12/06 21:31:54  timc
 * no changes
 *
 * Revision 1.4  2007/12/06 21:14:53  timc
 * code cleanup, use registry value to set service
 *
 * Revision 1.3  2007/11/26 17:19:59  timc
 * checkin prior to major cleanup
 *
 * Revision 1.2  2007/10/26 21:41:27  timc
 * k5 login working, no tickets stashed yet
 *
 * Revision 1.1  2007/10/24 20:33:57  timc
 * Initial revision
 *
 * Revision 1.2  2007/10/22 21:07:04  timc
 * cleanup prior to real work
 *
 * Revision 1.1  2007/10/22 19:58:03  timc
 * Initial revision
 *
 * 
 *
 */



/*
 *
 *  uwkfuncs.c
 *
 *  All kerberos/AFS related functions & utilities, and functions that refuse to be made 
 *  into unicode functions.
 * 
 *  Much of the codebase was written by Craig Huckabee, formerly CSL.  
 *
 */ 



#include <winsock2.h>

#include "stdafx.h"
#include "uwutil.h"


#define HAVE_KRB5_CREDS_KEYBLOCK

#include <krb5/krb5/krb5.h>
#include <krbcc/cacheapi.h>
#include <krb5/com_err.h>

#include <time.h>
#include <winreg.h>
#include <string>

#define	KRB5_DEFAULT_LIFE 60 * 60 * UWG_DEF_LIFETIME
#define AFSKEY "afs"
#define AFSINST ""



krb5_data tgtname = {
   0,
   KRB5_TGS_NAME_SIZE,
   KRB5_TGS_NAME
};

static const char *kadm_replies[] = {
  "Operation successful",             			/* KRB5_ADM_SUCCESS */
  "Command not recognized",           			/* KRB5_ADM_CMD_UNKNOWN */
  "Password unacceptable to server",  			/* KRB5_ADM_PW_UNACCEPT */
  "Old password incorrect",           			/* KRB5_ADM_BAD_PW */
  "Invalid ticket (TKT_FLAG_INITIAL not set)",	/* KRB5_ADM_NOT_IN_TKT */ 
  "Server refused password change",   			/* KRB5_ADM_CANT_CHANGE */
  "Language not supported",						/* KRB5_ADM_LANG_NOT_SUPPORTED */
};



static const char *kadm_replies_unknown = "UNKNOWN ERROR";
static char errbuf[1024];						/* For response from kadm */



static krb5_error_code keyadmin_send_receive(krb5_context,
					 int *,
					 krb5_auth_context,
					 krb5_int32,
					 krb5_data *,
					 krb5_int32 *,
					 krb5_int32 *,
					 krb5_data **);
static char * get_admin_response(krb5_int32,krb5_int32,krb5_data *);		
char * FlagsString(krb5_creds *);
time_t GetEpoch(void);
static char * short_date (long);



krb5_context kcontext;
krb5_creds my_creds;		
krb5_principal me;
WCHAR errortext[UWG_BIGBUFFER];
const char			*message; /* error string */
krb5_principal server;





/*
 * Function	: GetK5Ticket
 *
 * Summary	: Get a Kerberos V ticket with the given password
 * Reality      : use krb5_get_init_creds_password to see if the user's password is
 *                OK with the KDC; if so, pass true back to the pGina plugin, and 
 *                let KfW deal with getting tickets.  This deals only with authentication
 *                and not-at-all with authorization.
 */
bool GetK5Ticket(PTCHAR strUserName, PTCHAR strPassword, UINT iLifetime, BOOL verbose)
{	
	char kusername[UWG_MAX_USERID];
	char kpassword[UWG_DEFAULTLENGTH];
	char kservice[UWG_MAX_REALMLEN];
	krb5_deltat lifetime = KRB5_DEFAULT_LIFE;
	krb5_timestamp starttime = 0;
	krb5_deltat rlife = 0;
	int options = KDC_OPT_FORWARDABLE|KDC_OPT_RENEWABLE;
	krb5_get_init_creds_opt new_options;
	int errflg = 0;
	krb5_error_code code;
	krb5_timestamp now;
	krb5_address **addrs = (krb5_address **)0;
	krb5_preauthtype * preauth = NULL;
	char *client_name;
	HKEY hKey;

	
	DWORD dwSize;
	DWORD dwType=REG_SZ;
	char *pData;


	__try{

		//get the default realm from the registry
		//HKLM\SOFTWARE\pGina\k5plugin\DefaultService
		debug_print("GetK5Ticket - 0\n");

		if( RegOpenKeyEx(HKEY_LOCAL_MACHINE,
						TEXT("SOFTWARE\\pGina\\k5plugin"),	
						0,
						KEY_QUERY_VALUE, 
						&hKey) == ERROR_SUCCESS)
		{
			RegQueryValueEx(hKey, TEXT("DefaultService"), NULL, &dwType, NULL, &dwSize);
			pData = (char *)malloc(dwSize);
			RegQueryValueEx(hKey, TEXT("DefaultService"), NULL, &dwType, (LPBYTE)pData, &dwSize);
			debug_print("\tDefaultService is: %S\n",pData);
		} else {
			debug_print("HKLM\\SW\\pGina\\k5plugin does not exist -- failing by default!\n");
			return(FALSE);
		}

		RegCloseKey(hKey);

		debug_print("GetK5Ticket - 1 \n");

		/* convert unicode service, username & password to ascii */
		WideCharToMultiByte(CP_ACP, 0, strUserName, wcslen(strUserName)+1, kusername, sizeof(kusername), NULL, NULL);
		WideCharToMultiByte(CP_ACP, 0, strPassword, wcslen(strPassword)+1, kpassword, sizeof(kpassword), NULL, NULL);
		WideCharToMultiByte(CP_ACP, 0, (LPCWSTR)pData, wcslen((LPCWSTR)pData)+1, kservice, sizeof(kservice), NULL, NULL);

		debug_print("GetK5Ticket - 2 \n");

		code = krb5_init_context(&kcontext);
		if (code) {
			message = error_message(code);
			MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, message, -1, errortext, UWG_BIGBUFFER - 1 );
			doErrorMessage(errortext,MB_ICONEXCLAMATION,0);
			return(FALSE);
		}

		debug_print("GetK5Ticket - 3 \n");

		if ((code = krb5_timeofday(kcontext, &now))) {
			message = error_message(code);
			MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, message, -1, errortext, UWG_BIGBUFFER - 1 );
			doErrorMessage(errortext,MB_ICONEXCLAMATION,0);
			return(FALSE);
		}

		debug_print("GetK5Ticket - 4 \n");

		if ((code = krb5_parse_name (kcontext, kusername, &me))) {
			message = error_message(code);
			MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, message, -1, errortext, UWG_BIGBUFFER - 1 );
			doErrorMessage(errortext,MB_ICONEXCLAMATION,0);
			return(FALSE);
		}

		debug_print("GetK5Ticket - 5 \n");

		if ((code = krb5_unparse_name(kcontext, me, &client_name))) {
			message = error_message(code);
			MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, message, -1, errortext, UWG_BIGBUFFER - 1 );
			doErrorMessage(errortext,MB_ICONEXCLAMATION,0);
			return(FALSE);
		}

		debug_print("GetK5Ticket - 6 \n");
		memset((char *)&my_creds, 0, sizeof(my_creds));
		debug_print("GetK5Ticket - 7 \n");
		my_creds.client = me;

		if((code = krb5_build_principal_ext(kcontext, &server,
			krb5_princ_realm(kcontext, me)->length,
			krb5_princ_realm(kcontext, me)->data,
			tgtname.length, tgtname.data,
			krb5_princ_realm(kcontext, me)->length,
			krb5_princ_realm(kcontext, me)->data,
			0))) {
				message = error_message(code);
				MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, message, -1, errortext, UWG_BIGBUFFER - 1 );
				doErrorMessage(errortext,MB_ICONEXCLAMATION,0);
				return(FALSE);
		}
		debug_print("GetK5Ticket - 8 \n");

		my_creds.server = server;
		my_creds.times.starttime = 0;
		if(iLifetime > 0)
			lifetime = iLifetime;
		my_creds.times.endtime = now + lifetime;
		my_creds.times.renew_till = now + rlife;

		//needed for krb5_get_init_creds_password last argument
		krb5_get_init_creds_opt_init(&new_options);
		//renew for the "lifetime" setting
		krb5_get_init_creds_opt_set_tkt_life(&new_options, lifetime);
		krb5_get_init_creds_opt_set_renew_life(&new_options, 1);
		krb5_get_init_creds_opt_set_forwardable(&new_options, 1);
		krb5_get_init_creds_opt_set_proxiable(&new_options, 0);


		//ref krb5funcs.c:1521
		code = krb5_get_init_creds_password(kcontext, &my_creds, me, kpassword, NULL, 0, 0, kservice, &new_options);
		debug_print("GetK5Ticket - 9 \n");

		memset(kpassword, 0, sizeof(kpassword));

		debug_print("GetK5Ticket - 10 \n");

		//if code, bail, do cleanup
		if(code){
			if(code == KRB5KRB_AP_ERR_BAD_INTEGRITY){
				if (verbose) doErrorMessage(_T("Password Incorrect !"),0,0);
				return(FALSE);
			}else{
				message = error_message(code);
				MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, message, -1, errortext, UWG_BIGBUFFER - 1 );
				doErrorMessage(errortext,MB_ICONEXCLAMATION,0);
				return(FALSE);
			}
		}



		return(TRUE);
	}__except(TRUE){
		doErrorMessage(_T("GetK5Ticket - GPF!"),MB_ICONSTOP,0);
		return(FALSE);
	}
}

