/*
 * Copyright 2007 University of Wisconsin-Madison
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the University of Wisconsin-Madison not
 * be used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  The University of
 * Wisconsin-Madison makes no representations about the suitability of this
 * software for any purpose.  It is provided "as is" without express or
 * implied warranty.
 *
 * THE UNIVERSITY OF WISCONSIN-MADISON DISCLAIMS ALL WARRANTIES WITH REGARD TO
 * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS, IN NO EVENT SHALL THE UNIVERSITY OF WISCONSIN-MADISON BE LIABLE FOR
 * ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
 * CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 *	Tim Czerwonka					Reseacher
 *	timc@cs.wisc.edu				Computer Sciences Department
 *	http://www.cs.wisc.edu/~timc	University of Wisconsin - Madison
 * 	phone: 608-262-6610				1210 West Dayton Street
 *  								Madison, WI 53706
 *
 *
 *           Craig Huckabee                Associate Researcher
 * Internet: huck@cs.wisc.edu              Department of Computer Sciences
 *      URL: http://www.cs.wisc.edu/~huck  University of Wisconsin-Madison
 *    Phone: (608)262-0438               1210 West Dayton Street
 *      FAX: (608)262-9777                 Madison, WI   53706-1685
 */

/*
 *
 * $Id: uwutil.cpp,v 1.4 2007/12/06 21:32:17 timc Exp $
 *
 *
 * $Log: uwutil.cpp,v $
 * Revision 1.4  2007/12/06 21:32:17  timc
 * changed debug output filename
 *
 * Revision 1.3  2007/12/06 21:14:53  timc
 * code cleanup, use registry value to set service
 *
 * Revision 1.2  2007/10/26 21:41:27  timc
 * k5 login working, no tickets stashed yet
 *
 * Revision 1.1  2007/10/24 20:34:23  timc
 * Initial revision (name change)
 *
 * Revision 1.2  2007/10/22 21:07:20  timc
 * remove items not immediately necessary for pgina plugin
 *
 * Revision 1.1  2007/10/22 19:58:03  timc
 * Initial revision
 *
 *
 *
 */


/*
 * uwutil.c 
 *
 * Utility functions that are used in k5 code
 *
 */
#include "stdafx.h"
#include <windows.h>
#define RTN_OK 0
#define RTN_ERROR 13
#define MY_BUFSIZE 1024 
#include "uwutil.h"










/*
 *
 * Function         : do_debug_print
 *
 * Summary         : Debug print to file function.
 *                 
 *
 */

void debug_print(char *message,...)
{
	static FILE *fh;
	char buffer[40];  
	SYSTEMTIME stime, stUTC;

	va_list arg_list;
	va_start (arg_list, message);				


	if(fh == NULL) {	
		char filename[100];
		srand((unsigned)time(NULL));

		sprintf(filename, "C:\\WINDOWS\\TEMP\\K5PGINA-%.4i",rand());

		fh = fopen(filename,"w+");
		if(!fh)
		return;
	}


	//prepend the current time to the uwgina log file
    GetSystemTime(&stUTC);
	//convert system time (in UTC) to local time
    SystemTimeToTzSpecificLocalTime(NULL, &stUTC, &stime);

    sprintf(buffer, "%02i-%02i-%0004i %02i:%02i:%02i \0", stime.wMonth, stime.wDay, stime.wYear, stime.wHour, stime.wMinute, stime.wSecond);

    vfprintf(fh, buffer, 0);

	if(vfprintf(fh,message,arg_list) < 0 )
		return;
	fflush(fh);
	return;
}




/*
 *
 * Function         : doErrorMessage
 *
 * Summary         : This is the error handle/display routine
 *                 
 *
 */

VOID doErrorMessage(PWSTR pszMessage, DWORD icon, DWORD code)
{
   WCHAR *title = _T("CSL Windows XP - Error!");   
   

   if(icon == 0)
      icon = MB_ICONEXCLAMATION;
   if(code != 0){
      
      LPVOID lpMsgBuf;

      FormatMessage( 
      FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
      NULL,
      GetLastError(),
      MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), /* Default language */
      (LPTSTR) &lpMsgBuf,
      0,
      NULL);


      LocalFree( lpMsgBuf );
      return;
   }

   debug_print("AN ERROR IS BEING DISPLAYED!\n");
   debug_print("\t%S\n", pszMessage);


   return;
}
